      double precision function ft11 (gx,gy)
c
c     Fourier transform of dielectric function at reciprocal
c     lattice vectors (gx,gy), for the case of
c     cylinders of dielectric constant eps1 
c     in a unit cell of dielectric constant eps2.
c     A square or triangular lattice is assumed for the condition 
c     of non-overlapping rods.
c
c     IT ASSUMES THAT (GX,GY) IS A RECIPROCAL LATTICE VECTOR,
c     OTHERWISE THE RESULTS IS INCORRECT.
c
      implicit none
      integer ifail
      real*8 g,eps1,eps2,radius,pi,area,aj1,gx,gy,bessj1,dummy3
      real*8 add,spher0,a1(2),a2(2),ucarea,aret,fill,dummy1,dummy2
      parameter (aret=1d0)
      common/dirlattice/a1,a2,ucarea
      common/eps/eps1,eps2,fill
      common/length/radius,dummy1,dummy2,dummy3
      pi=4d0*atan(1d0)
      if (radius.gt.aret/2d0) stop ' from ft11: overlapping rods'
      area=pi*radius*radius
      fill=area/ucarea
      if (gx.eq.0.and.gy.eq.0) then
        ft11=fill*eps1+(1d0-fill)*eps2
      else
        g=sqrt(gx*gx+gy*gy)
        ifail=0
!        aj1=s17aff(g*radius,ifail)
        aj1=bessj1(g*radius)
        ft11=(eps1-eps2)/ucarea*2d0*pi*radius/g*aj1
c
c       In the particular case of a square lattice,
c       the Fourier transform can be made valid even if
c       (gx,gy) is not a reciprocal lattice vector
c       by adding the following term:
c
!        add=eps2*spher0(gx/2d0)*spher0(gy/2d0)
!        print *, 'add=',add
!        ft11=ft11+add
      endif
      return
      end
c
      double precision function ft12 (gx,gy)
c
c     Fourier transform of dielectric function at reciprocal
c     lattice vectors (gx,gy), for the case of
c     square rods of dielectric constant eps1 (non rotated) 
c     in a unit cell of dielectric constant eps2.
c
c     Is valid for any type of 2D Bravais lattice,
c     provided the unit cell area is given.
c     
c     IT ASSUMES THAT (GX,GY) IS A RECIPROCAL LATTICE VECTOR,
c     OTHERWISE THE RESULTS IS INCORRECT.
c
      implicit none
      integer ifail
      real*8 eps1,eps2,side,area,gx,gy
      real*8 add,spher0,a1(2),a2(2),ucarea,fill,dummy1,dummy2,dummy3
      common/dirlattice/a1,a2,ucarea
      common/eps/eps1,eps2,fill
      common/length/side,dummy1,dummy2,dummy3
      area=side*side
      fill=area/ucarea
      if (gx.eq.0.and.gy.eq.0) then
        ft12=fill*eps1+(1d0-fill)*eps2
      else
        ft12=(eps1-eps2)*area/ucarea*
     &      spher0(gx*side/2d0)*spher0(gy*side/2d0)
      endif
      return
      end
c
      double precision function ft13 (gx,gy)
c
c     Fourier transform of dielectric function at reciprocal
c     lattice vectors (gx,gy), for the case of
c     square rods of dielectric constant eps1 rotated by 45 deg
c     in a unit cell of dielectric constant eps2.
c     It is the chess-board lattice.
c
c     Is valid for any type of 2D Bravais lattice,
c     provided the unit cell area is given.
c     
c     IT ASSUMES THAT (GX,GY) IS A RECIPROCAL LATTICE VECTOR,
c     OTHERWISE THE RESULTS IS INCORRECT.
c
      implicit none
      integer ifail
      real*8 eps1,eps2,side,area,gx,gy,diag
      real*8 add,spher0,a1(2),a2(2),ucarea,fill,dummy1,dummy2,dummy3
      common/dirlattice/a1,a2,ucarea
      common/eps/eps1,eps2,fill
      common/length/side,dummy1,dummy2,dummy3
      area=side*side
      diag=sqrt(2d0)*side
      fill=area/ucarea
      if (gx.eq.0.and.gy.eq.0) then
        ft13=fill*eps1+(1d0-fill)*eps2
      else if (gx.eq.gy.or.gx.eq.-gy) then
        ft13=(eps1-eps2)*diag/ucarea*sin(gx*diag/2d0)/gx
      else
        ft13=4d0*(eps1-eps2)/ucarea
     &     *(cos(gy*diag/2d0)-cos(gx*diag/2d0))/(gx*gx-gy*gy)
      endif
      return
      end
c
      double precision function ft99 (gx,gy)
c
c     Fourier transform of dielectric function at reciprocal
c     lattice vectors (gx,gy), for the case of stripes (1D).
c     The lattice is periodic along x and homogeneous along y,
c     thus the Fourier transform is nonzero only if gy=0.
c
c     IT ASSUMES THAT (GX,GY) IS A RECIPROCAL LATTICE VECTOR,
c     OTHERWISE THE RESULTS IS INCORRECT.
c
      implicit none
      integer ifail
      real*8 eps1,eps2,gx,gy,alength1,dum1,dum2,dum3
      real*8 spher0,a1(2),a2(2),ucarea,fill
      common/dirlattice/a1,a2,ucarea
      common/eps/eps1,eps2,fill
      common/length/alength1,dum1,dum2,dum3
      if (alength1.gt.1) stop ' from ft99: overlapping stripes'
      if (gy.eq.0) then
        if (gx.eq.0) then
          ft99=alength1*eps1+(1d0-alength1)*eps2
        else
          ft99=(eps1-eps2)*alength1*spher0(gx*alength1/2d0)
        endif
      else
        ft99=0d0
      endif
      return
      end
c
      double precision function ft98 (gx,gy)
c
c     Fourier transform of dielectric function at reciprocal
c     lattice vector (gx,gy), for the case of cavity embedded between 
c     DBR mirrors with supercell repetition.
c     The lattice is periodic along x and homogeneous along y,
c     thus the Fourier transform is nonzero only if gy=0.
c
c     Notations are as follows:
c     
c     layer 1=air         (dielectric constant eps1)
c     layer 2=dielectric  (dielectric constant eps2)
c     layer c=cavity      (dielectric constant epsc=eps2)
c
c     alength1  =thickness of air layer, in unit of DBR period
c     1-alength1=thickness of dielectric layer, in unit of DBR period
c     alengthc  =thickness of cavity layer, in unit of DBR period
c     an1=n1  =number of air        layers between two cavities (integer #)
c     n2 =n1-1=number of dielectric layers between two cavities 
c     al1=thickness of air        layer,  in units of supercell period
c     al2=thickness of dielectric layers, in units of supercell period
c     alc=thickness of cavity     layer,  in units of supercell period
c
c     The supercell period=alc+n1*al1+n2*al2 is normalized to one.
c
c     IT ASSUMES THAT GX,GY IS A RECIPROCAL LATTICE VECTOR,
c     OTHERWISE THE RESULTS IS INCORRECT.
c
      implicit none
      integer ifail,n1,n2,k
      real*8 eps1,eps2,epsc,gx,gy,alength1,alengthcav,an1,spher0
      real*8 al1,al2,alc,supercellperiod,dbrperiod,argum,upper
      real*8 a1(2),a2(2),ucarea,fill,dummy
      complex*16 i,fstruc
      parameter (i=(0d0,1d0),upper=1d-12)
      common/dirlattice/a1,a2,ucarea
      common/eps/eps1,eps2,fill
      common/length/alength1,alengthcav,an1,dummy

      if (alength1.gt.1) stop ' from ft98: overlapping stripes'
      n1=int(an1)
      if (n1.ne.an1) stop ' from ft98: alength3 must be integer'
      n2=n1-1
      epsc=eps2

      if (gy.eq.0) then 
c
c      normalized lengths to supercell period
c
       supercellperiod=n1*alength1+n2*(1d0-alength1)+alengthcav
       al1=alength1      /supercellperiod
       al2=(1d0-alength1)/supercellperiod
       alc=alengthcav    /supercellperiod
       dbrperiod=al1+al2
       fill=alc+n2*al2
       if (gx.eq.0) then
        ft98=alc*epsc+n1*al1*eps1+n2*al2*eps2
       else
c
c       Structure factor of dielectric regions. There are two cases:
c       1) n1 odd : the sum extends from k=1,3,5,...,n2-1 with +/- terms
c       2) n1 even: the sum extends from k=1,3,5,...,n2-2 with +/- terms
c                   and an additional term with k=n2
c       In both cases, the total number of terms in the sum
c       is equal to n2=number of dielectric layers.
c
        fstruc=(0d0,0d0)
        do k=1,n2-1,2
          argum=gx/2d0*(alc+al1+k*dbrperiod)
          fstruc=fstruc+exp(i*argum)+exp(-i*argum)
        enddo
        if (2*(n1/2).eq.n1) then               ! additional term for even n1
          argum=gx/2d0*(alc+al1+n2*dbrperiod)
          fstruc=fstruc+exp(i*argum)
        endif
        if (dimag(fstruc).gt.upper) stop 'from ft98: complex FSTRUC'
        ft98=(epsc-eps1)*alc*spher0(gx*alc/2d0)
     &      +(eps2-eps1)*al2*spher0(gx*al2/2d0)*dble(fstruc)
       endif
      else
        ft98=0d0
      endif
      return
      end
c
      double precision function ft22 (gx,gy)
c
c     Fourier transform of dielectric function at reciprocal
c     lattice vectors (gx,gy), for the case of GRAPHITE STRUCTURE
c     with two cylinders of dielectric constant eps1 
c     in a unit cell of dielectric constant eps2.
c
c     IT ASSUMES THAT (GX,GY) IS A RECIPROCAL LATTICE VECTOR,
c     OTHERWISE THE RESULTS IS INCORRECT.
c
      implicit none
      integer ifail
      real*8 g,eps1,eps2,radius,pi,area,aj1,gx,gy,dummy1,bessj1,dummy3
      real*8 add,spher0,a1(2),a2(2),ucarea,aret,strucfac,fill,dummy2
      parameter (aret=1d0)
      common/dirlattice/a1,a2,ucarea
      common/eps/eps1,eps2,fill
      common/length/radius,dummy1,dummy2,dummy3
      pi=4d0*atan(1d0)
      if (radius.gt.aret/sqrt(12d0)) stop ' from ft22: overlapping rods'
      area=2d0*pi*radius*radius
      fill=area/ucarea
      if (gx.eq.0.and.gy.eq.0) then
        ft22=fill*eps1+(1d0-fill)*eps2
      else
        g=sqrt(gx*gx+gy*gy)
        ifail=0
!        aj1=s17aff(g*radius,ifail)
        aj1=bessj1(g*radius)
        strucfac=2d0*cos(gy*aret/2d0/sqrt(3d0))
        ft22=(eps1-eps2)/ucarea*2d0*pi*radius/g*aj1*strucfac
      endif
      return
      end
c
      double precision function ft40 (gx,gy)

c     written by L.C. Andreani, 05/09/02
c     Fourier transform of dielectric function at reciprocal
c     lattice vectors (gx,gy), for the case of WAVEGUIDE TYPE W1
c     with cylinders of dielectric constant eps1 
c     in a unit cell of dielectric constant eps2.
c
c     The channel thickness is denoted by w 
c     and equals w0=a*sqrt(3) for a W1 waveguide.
c
c     Used for calculation of bulk 2D, but with supercell of W1 waveguide.
c     Derived from ft41, by adding holes at positions of channel waveguide.
c
c     alength1=hole radius
c     alength2=Waveguide's width or channel thickness (in units of w0): 
c              twice the distance between the center of the supercell 
c              and the center of the nearest cylinder.
c              For a W1 waveguide, alength2=1.
c     alength3=integer number, which yields the supercell width
c              in the direction perpendicular to the waveguide 
c              as (alength2+alength3)*w0. In the W1 case, two waveguides 
c              are separated by (alength3+1) periods.
c
c     IT ASSUMES THAT (GX,GY) IS A RECIPROCAL LATTICE VECTOR,
c     OTHERWISE THE RESULTS IS INCORRECT.

      implicit none
      integer l3,lbar,nholes,n
      real*8 radius,w,w0,pi,aret,gx,gy,g,check
      real*8 aj1,bessj1,area,ucarea,a1(2),a2(2)
      real*8 eps1,eps2,fill,alength2,alength3,dummy,upper
      complex*16 geofull,geohalf,geoquarter,geostruc,i
      real*8 holestruc
      parameter (aret=1d0,i=(0d0,1d0),upper=1d-12)

      common/dirlattice/a1,a2,ucarea
      common/eps/eps1,eps2,fill
      common/length/radius,alength2,alength3,dummy

      if (radius.gt.aret/2d0) stop 'from ft40: overlapping holes'
      if (radius.gt.sqrt(3d0)/2d0*aret*alength2)
     &  stop 'from ft40: overlapping holes'
      l3=int(alength3)
      if (dble(l3).ne.alength3) stop 
     &    ' from ft40: alength3 must be an integer'

      pi=4d0*atan(1d0)
      w0=sqrt(3d0)*aret
      w=alength2*w0
      check=(w+l3*w0)*aret
      if (abs(check-ucarea).gt.upper) stop 'from ft40: wrong ucarea'
      nholes=2*l3+1
      nholes=nholes+1  ! ONE MORE HOLE FOR BULK 2D
      area=pi*radius*radius
      fill=nholes*area/ucarea

      if (gx.eq.0d0.and.gy.eq.0d0) then
        ft40=eps1*fill+eps2*(1d0-fill)
      else
c
c       calculation of geometrical structure factor
c
        geofull=(0d0,0d0)
        geohalf=(0d0,0d0)
        geoquarter=(0d0,0d0)
        if ((l3/2)*2.eq.l3) then  ! even l3: full and half holes
          lbar=l3/2

          do n=0,lbar-1
            geofull=geofull+2d0*dble(exp(i*gy*(w/2d0+n*w0)))
            geohalf=geohalf+exp(i*gx*aret/2d0)*
     &                 2d0*dble(exp(i*gy*((w+w0)/2d0+n*w0)))
          enddo
          geohalf=geohalf+exp(i*gy*(w+l3*w0)/2d0) ! two more half holes

        else                      ! odd l3: full, half and quarter holes
          lbar=(l3-1)/2

          do n=0,lbar
            geofull=geofull+2d0*dble(exp(i*gy*(w/2d0+n*w0)))
          enddo

          do n=0,lbar-1
            geohalf=geohalf+exp(i*gx*aret/2d0)*
     &                 2d0*dble(exp(i*gy*((w+w0)/2d0+n*w0)))
          enddo

          geoquarter=exp(i*gx*aret/2d0)*exp(i*gy*(w+l3*w0)/2d0)

        endif
        geohalf=geohalf+exp(i*gx*aret/2d0)   ! PUT HOLES IN CHANNEL FOR BULK 2D
        geostruc=geofull+geohalf+geoquarter
        if (dimag(geostruc).gt.upper) stop 
     &    'from ft40: geometrical structure factor is not real'
c
c       calculation of hole structure factor
c

        g=sqrt(gx*gx+gy*gy)
        aj1=bessj1(g*radius)
        holestruc=2d0*pi*radius/g*aj1
c
c       calculation of Fourier transform
c
        ft40=(eps1-eps2)/ucarea*dble(geostruc)*holestruc
      endif
      return
      end
c
      double precision function ft41 (gx,gy)

c     written by L.C. Andreani, 05/09/02
c     Fourier transform of dielectric function at reciprocal
c     lattice vectors (gx,gy), for the case of WAVEGUIDE TYPE W1
c     with cylinders of dielectric constant eps1 
c     in a unit cell of dielectric constant eps2.
c
c     The channel thickness is denoted by w=alength2*w0
c     and equals w0=a*sqrt(3) for a W1 waveguide.
c
c     alength1=hole radius
c     alength2=Waveguide's width or channel thickness (in units of w0): 
c              twice the distance between the center of the supercell 
c              and the center of the nearest cylinder.
c              For a W1 waveguide, alength2=1.
c     alength3=integer number, which yields the supercell width
c              in the direction perpendicular to the waveguide 
c              as (alength2+alength3)*w0. In the W1 case, two waveguides 
c              are separated by (alength3+1) periods.
c
c     IT ASSUMES THAT (GX,GY) IS A RECIPROCAL LATTICE VECTOR,
c     OTHERWISE THE RESULTS IS INCORRECT.

      implicit none
      integer l3,lbar,nholes,n
      real*8 radius,w,w0,pi,aret,gx,gy,g,check
      real*8 aj1,bessj1,area,ucarea,a1(2),a2(2)
      real*8 eps1,eps2,fill,alength2,alength3,upper,dummy
      complex*16 geofull,geohalf,geoquarter,geostruc,i
      real*8 holestruc
      parameter (aret=1d0,i=(0d0,1d0),upper=1d-12)

      common/dirlattice/a1,a2,ucarea
      common/eps/eps1,eps2,fill
      common/length/radius,alength2,alength3,dummy

      if (radius.gt.aret/2d0) stop 'from ft41: overlapping holes'
      if (radius.gt.sqrt(3d0)/2d0*aret*alength2)
     &  stop 'from ft41: overlapping holes'
      l3=int(alength3)
      if (dble(l3).ne.alength3) stop 
     &    ' from ft41: alength3 must be an integer'

      pi=4d0*atan(1d0)
      w0=sqrt(3d0)*aret
      w=alength2*w0
      check=(w+l3*w0)*aret
      if (abs(check-ucarea).gt.upper) stop 'from ft41: wrong ucarea'
      nholes=2*l3+1
      area=pi*radius*radius
      fill=nholes*area/ucarea

      if (gx.eq.0d0.and.gy.eq.0d0) then
        ft41=eps1*fill+eps2*(1d0-fill)
      else
c
c       calculation of geometrical structure factor
c
        geofull=(0d0,0d0)
        geohalf=(0d0,0d0)
        geoquarter=(0d0,0d0)
        if ((l3/2)*2.eq.l3) then  ! even l3: full and half holes
          lbar=l3/2

          do n=0,lbar-1
            geofull=geofull+2d0*dble(exp(i*gy*(w/2d0+n*w0)))
            geohalf=geohalf+exp(i*gx*aret/2d0)*
     &                 2d0*dble(exp(i*gy*((w+w0)/2d0+n*w0)))
          enddo
          geohalf=geohalf+exp(i*gy*(w+l3*w0)/2d0) ! two more half holes

        else                      ! odd l3: full, half and quarter holes
          lbar=(l3-1)/2

          do n=0,lbar
            geofull=geofull+2d0*dble(exp(i*gy*(w/2d0+n*w0)))
          enddo

          do n=0,lbar-1
            geohalf=geohalf+exp(i*gx*aret/2d0)*
     &                 2d0*dble(exp(i*gy*((w+w0)/2d0+n*w0)))
          enddo

          geoquarter=exp(i*gx*aret/2d0)*exp(i*gy*(w+l3*w0)/2d0)

        endif
        geostruc=geofull+geohalf+geoquarter
        if (dimag(geostruc).gt.upper) stop 
     &    'from ft41: geometrical structure factor is not real'
c
c       calculation of hole structure factor
c

        g=sqrt(gx*gx+gy*gy)
        aj1=bessj1(g*radius)
        holestruc=2d0*pi*radius/g*aj1
c
c       calculation of Fourier transform
c
        ft41=(eps1-eps2)/ucarea*dble(geostruc)*holestruc
      endif
      return
      end
c
